@@ -13,9 +13,9 @@ contents = open('python36.sdef').read().replace("\n", "").split(';;')
 
 func_structure = '''
     Ast* visitNode(%{RULE_FOR}* node) {
-        if ( ! node ) return 0;
+        if ( ! node ) return nullptr;
         bool ranges_copied = false; Q_UNUSED(ranges_copied);
-        Ast* result = 0;
+        Ast* result = nullptr;
         switch ( node->kind ) {
 %{SWITCH_LINES}
         default:
@@ -37,7 +37,7 @@ func_structure = '''
 simple_func_structure = '''
     Ast* visitNode(%{RULE_FOR}* node) {
         bool ranges_copied = false; Q_UNUSED(ranges_copied);
-        if ( ! node ) return 0;
+        if ( ! node ) return nullptr;
 %{SWITCH_LINES}
         return v;
     }
@@ -50,10 +50,10 @@ switch_line = '''        case %{KIND}: {
             }'''
 
 create_ast_line = '''                %{AST_TYPE}* v = new  %{AST_TYPE}(parent());'''
-create_identifier_line = '''                v->%{TARGET} = node->v.%{KIND_W/O_SUFFIX}.%{VALUE} ? new Python::Identifier(PyUnicodeObjectToQString(node->v.%{KIND_W/O_SUFFIX}.%{VALUE})) : 0;'''
+create_identifier_line = '''                v->%{TARGET} = node->v.%{KIND_W/O_SUFFIX}.%{VALUE} ? new Python::Identifier(PyUnicodeObjectToQString(node->v.%{KIND_W/O_SUFFIX}.%{VALUE})) : nullptr;'''
 set_attribute_line = '''                nodeStack.push(v); v->%{TARGET} = static_cast<%{AST_TYPE}*>(visitNode(node->v.%{KIND_W/O_SUFFIX}.%{VALUE})); nodeStack.pop();'''
 resolve_list_line = '''                nodeStack.push(v); v->%{TARGET} = visitNodeList<%{PYTHON_AST_TYPE}, %{AST_TYPE}>(node->v.%{KIND_W/O_SUFFIX}.%{VALUE}); nodeStack.pop();'''
-create_identifier_line_any = '''            v->%{TARGET} = node->%{VALUE} ? new Python::Identifier(PyUnicodeObjectToQString(node->%{VALUE})) : 0;'''
+create_identifier_line_any = '''            v->%{TARGET} = node->%{VALUE} ? new Python::Identifier(PyUnicodeObjectToQString(node->%{VALUE})) : nullptr;'''
 set_attribute_line_any = '''            nodeStack.push(v); v->%{TARGET} = static_cast<%{AST_TYPE}*>(visitNode(node->%{VALUE})); nodeStack.pop();'''
 resolve_list_line_any = '''            nodeStack.push(v); v->%{TARGET} = visitNodeList<%{PYTHON_AST_TYPE}, %{AST_TYPE}>(node->%{VALUE}); nodeStack.pop();'''
 direct_assignment_line = '''                v->%{TARGET} = node->v.%{KIND_W/O_SUFFIX}.%{VALUE};'''
@@ -122,13 +122,18 @@ for rule in contents:
     actions = outline[2].split(' ')[1:]
     code = None
     since_version = None
+    before_version = None
     if len(outline) > 3:
-        if outline[3].startswith('SINCE'):
+        if outline[3].startswith('BEFORE'):
+            before_version = [int(n) for n in outline[3][7:].split('.')]
+        elif outline[3].startswith('SINCE'):
             since_version = [int(n) for n in outline[3][6:].split('.')]
         elif outline[3].startswith('CODE'):
             code = ' '.join(';'.join(outline[3:]).split('CODE')[1:]) + ";"
         else:
             raise SyntaxError('Invalid syntax in sdef file, line: ' + rule)
+    if len(outline) > 4 and outline[4].startswith('CODE'):
+        code = ' '.join(';'.join(outline[4:]).split('CODE')[1:]) + ";"
     
     if rule_for not in results:
         results[rule_for] = list()
@@ -208,8 +213,12 @@ for rule in contents:
         current_stmt = current_actions
     else:
         current_stmt = switch_line.replace('%{KIND}', kind).replace('%{ACTIONS}', current_actions)
+    if before_version:
+        version_cpp_if = ("#if PYTHON_VERSION < QT_VERSION_CHECK(%d, %d, 0)\n"
+                           %(before_version[0], before_version[1]))
+        current_stmt = version_cpp_if + current_stmt + "\n#endif"
     if since_version:
-        version_cpp_if = ("#if PYTHON_VERSION_MAJOR >= %d && PYTHON_VERSION_MINOR >= %d\n"
+        version_cpp_if = ("#if PYTHON_VERSION >= QT_VERSION_CHECK(%d, %d, 0)\n"
                            %(since_version[0], since_version[1]))
         current_stmt = version_cpp_if + current_stmt + "\n#endif"
     results[rule_for].append(current_stmt)
@@ -226,7 +235,6 @@ print('''/* This code is generated by conversiongenerator.py.
 class PythonAstTransformer {
 public:
     CodeAst* ast;
-    PythonAstTransformer(int lineOffset) : m_lineOffset(lineOffset) {};
     void run(mod_ty syntaxtree, QString moduleName) {
         ast = new CodeAst();
         ast->name = new Identifier(moduleName);
@@ -241,12 +249,11 @@ public:
             // don't touch the marker
             return -99999;
         }
-        return line + m_lineOffset;
+        return line;
     };
 private:
     QStack<Ast*> nodeStack;
-    int m_lineOffset;
-    
+
     Ast* parent() {
         return nodeStack.top();
     }
@@ -270,7 +277,7 @@ for index, lines in sorted(results.items()):
     appendix = ''
     if index == '_expr' or index == '_stmt':
         appendix = '''
-	if ( ! result ) return 0;
+	if ( ! result ) return nullptr;
         if ( ! ranges_copied ) {
             result->startCol = node->col_offset;
             result->endCol = node->col_offset;
