@@ -19,7 +19,7 @@
 #ifndef PYTHON_PARSESESSION_H
 #define PYTHON_PARSESESSION_H
 
-#include <QtCore/QString>
+#include <QString>
 
 #include "parserexport.h"
 #include <language/duchain/duchainpointer.h>
@@ -46,7 +46,7 @@ class KDEVPYTHONPARSER_EXPORT ParseSession : public IAstContainer
 {
 public:
     ParseSession();
-    ~ParseSession();
+    ~ParseSession() override;
 
     void setContents( const QString& contents );
     QString contents() const;
@@ -61,12 +61,6 @@ public:
     const ModificationRevision& futureModificationRevision() const;
     void setFutureModificationRevision(const ModificationRevision& revision);
     
-    void mapAstUse(Ast* node, const SimpleUse& use)
-    {
-        Q_UNUSED(node);
-        Q_UNUSED(use);
-    }
-    
     CodeAst::Ptr ast;
     
 private:
